// ---------------------------------------------

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import gui.CircuitPanel;
import gui.CircuitPanelListener;
import gui.Label;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.FileNotFoundException;

import javax.swing.JFrame;
import javax.swing.JComboBox;

import parser.LTLSpecification;
import parser.VerilogToSMVConverter;

// ---------------------------------------------

public class Main extends JFrame
{
	public static final int CLIENT_WIDTH = 500;
	
	public static final int CLIENT_HEIGHT = 540;
	
	public static final int LBLEXAMPLES_HEIGHT = 20;
	
	public static final int CMBEXAMPLES_HEIGHT = 20;
	
	protected Label lblExamples;
	
	protected Label lblCircuits;
	
	public JComboBox cmbExamples;	// Per selezionare una tipologia di circuito.
	
	public JComboBox cmbCircuits;	// Per selezionare uno specifico circuito, tra quelli disponibili per la tipologia selezionata.
	
	protected CircuitPanel pnlCircuit;
	
	public static final File mainDir = ldir.localDir.getLocalDirRef(Main.class); // TM
	public static final File workingDirectory = new File( mainDir.getParentFile(), "examples" );  // TM
	
    public static void main( String[] args ) throws FileNotFoundException
    {     
    	new Main();
    	
    	// NOTA: il convertitore pu eseguire side-effect sulla specifica LTL, qualora sia presente
    	// almeno un inverter 3-state.
    	  
        //VerilogToSMVConverter converter = new VerilogToSMVConverter( "..\\examples\\example.vlg", "..\\examples\\example.smv", new LTLSpecification() );
    }
    
    public Main()
    {
    	initializeWindow();
    	
    	initializeGUIComponents();
    	
    	repaint();
    }
    
    private void initializeWindow()
    {
    	setTitle( "Tesina MFIS - A. A. 2006/07 - Verifica formale di circuiti digitali" );
    	
    	setResizable( false );
    	
    	setVisible( true );
    	
    	setSize( CLIENT_WIDTH + getInsets().left + getInsets().right, CLIENT_HEIGHT + getInsets().top + getInsets().bottom );
    	
    	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    	
    	setLocation( ( screen.width - getSize().width ) / 2, ( screen.height - getSize().height ) / 2 );
    
    	getContentPane().setBackground( Color.WHITE );
    	
    	setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
    	
    	setLayout( null );    	
    }
    
    private void initializeGUIComponents()
    {
    	lblExamples = new Label( ( CLIENT_WIDTH - CircuitPanel.PANEL_WIDTH ) / 2, ( CLIENT_HEIGHT - CircuitPanel.PANEL_HEIGHT - 2 * LBLEXAMPLES_HEIGHT - 2 * CMBEXAMPLES_HEIGHT ) / 6, CircuitPanel.PANEL_WIDTH, LBLEXAMPLES_HEIGHT, "Selezionare un tipo di circuito, tra i seguenti disponibili:" );
    	
    	lblExamples.setFont( new Font( "Verdana", 0, 10 ) );
    	
    	add( lblExamples );
    	
    	
    	lblCircuits = new Label( lblExamples.getLocation().x, 3 * lblExamples.getLocation().y + LBLEXAMPLES_HEIGHT + CMBEXAMPLES_HEIGHT, CircuitPanel.PANEL_WIDTH, LBLEXAMPLES_HEIGHT, "Selezionare uno specifico circuito, tra i seguenti disponibili per il tipo selezionato:" );
    	
    	lblCircuits.setFont( lblExamples.getFont() );
    	
    	add( lblCircuits );
    	
    	
    	pnlCircuit = new CircuitPanel( lblExamples.getLocation().x, 5 * lblExamples.getLocation().y + 2 * LBLEXAMPLES_HEIGHT + 2 * CMBEXAMPLES_HEIGHT, cmbExamples, cmbCircuits );
    	
    	pnlCircuit.setCircuitPanelListener
    	(
    		new CircuitPanelListener()
    		{
    			public void verificationStarted()
    			{
    				cmbExamples.setEnabled( false );
    				
    				cmbCircuits.setEnabled( false );
    			}
    			
    			public void verificationFinished()
    			{
    				cmbExamples.setEnabled( true );
    				
    				cmbCircuits.setEnabled( true );
    			}
    		}
    	);
    	    	
    	add( pnlCircuit );
    	
    	
    	cmbExamples = new JComboBox();
    	
    	cmbExamples.setBounds( lblExamples.getLocation().x, 2 * lblExamples.getLocation().y + LBLEXAMPLES_HEIGHT, CircuitPanel.PANEL_WIDTH, CMBEXAMPLES_HEIGHT );
    	
    	cmbExamples.setFont( lblExamples.getFont() );
    	
    	cmbExamples.setBackground( new Color( 230, 240, 255 ) );
    	
    	cmbExamples.addActionListener( new CircuitSelectionListener( this ) );
    	
    	add( cmbExamples );
    	
    	
    	cmbCircuits = new JComboBox();
    	
    	cmbCircuits.setBounds( lblExamples.getLocation().x, 4 * lblExamples.getLocation().y + 2 * LBLEXAMPLES_HEIGHT + CMBEXAMPLES_HEIGHT, CircuitPanel.PANEL_WIDTH, CMBEXAMPLES_HEIGHT );
    	
    	cmbCircuits.setFont( lblExamples.getFont() );
    	
    	cmbCircuits.setBackground( new Color( 230, 240, 255 ) );
    	
    	cmbCircuits.addActionListener( cmbExamples.getActionListeners()[ 0 ] );
    	
    	add( cmbCircuits );
    	
    	initializeInformations();    	
    }
    
    private void initializeInformations()
    {
    	//File workingDirectory = new File( localDir.getParentFile(), "examples" );  // TM
    			
    	FileFilter filter_1 = new FileFilter()
    	{
        	public boolean accept( File file )
        	{            
            	return file.isDirectory();
        	}
    	};
    	
    	File[] contents = workingDirectory.listFiles( filter_1 );
    	    	
    	if ( contents != null )
    	{
    		for ( int i = 0; i < contents.length; i++ )
    			
    			cmbExamples.addItem( contents[ i ].getName() );
    	}
    	
    	else
    	{
    		cmbExamples.addItem( "[ nessun tipo disponibile ]" );
    	}    	
    }
}

class CircuitSelectionListener implements ActionListener
{
	private Main main;
	
	public CircuitSelectionListener( Main main )
	{
		this.main = main;
	}
	
	public void actionPerformed( ActionEvent e )
	{
		Object obj = e.getSource();
		
		if ( obj.equals( main.cmbExamples ) )
		{
			main.cmbCircuits.removeAllItems();
			
			FilenameFilter filter = new FilenameFilter()
	    	{
	    		public boolean accept( File file, String name )
	        	{            
	            	return name.endsWith( ".vlg" );
	        	}
	    	};
	    	
	    	File[] contents = (new File( Main.workingDirectory, /*"../examples/" +*/  main.cmbExamples.getSelectedItem().toString() )).listFiles( filter ); // TM
	    	
	    	if ( contents != null )
	    	{
	    		for ( int i = 0; i < contents.length; i++ )
	    			
	    			main.cmbCircuits.addItem( contents[ i ].getName() );
	    	}
	    	
	    	else
	    	{
	    		main.cmbCircuits.addItem( "[ nessun circuito disponibile per il tipo selezionato ]" );
	    	}
		}
		
		else if ( obj.equals( main.cmbCircuits ) )
		{
			if ( main.cmbCircuits.getSelectedItem() != null )
				
				main.pnlCircuit.loadCircuit(  (  new File(Main.workingDirectory, /*"../examples/" +*/ 
														   main.cmbExamples.getSelectedItem() + 
															File.separator + main.cmbCircuits.getSelectedItem()
														 )
											  ).toString() 
				); // TM
		}
	}
}
