package gui;

import circuit.CircuitComponent;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;

import javax.swing.table.AbstractTableModel;

import parser.VerilogParser;
import parser.VerilogToSMVConverter;
import parser.LTLSpecification;


public class CircuitPanel extends JPanel
{
	public static final int PANEL_WIDTH = 470;
	
	public static final int PANEL_HEIGHT = 430;
	
	public static final int LBLDESCRIPTION_WIDTH = 100;
	
	public static final int LBLDESCRIPTION_HEIGHT = 15;
	
	public static final int VSPACE = 10;
	
	public static final int TBLCOMPONENTS_WIDTH = 12 * PANEL_WIDTH / 20;
	
	public static final int TBLCOMPONENTS_HEIGHT = 100;
	
	public static final int TBLINPUTS_WIDTH = 65;
	
	public static final int TBLCOMPONENTINPUTS_WIDTH = 95;
	
	public static final int CHKBMC_WIDTH = 197;
	
	public static final int SPNDISTANCE = 0;
	
	public static final int TXTSPECIFICATION_HEIGHT = 90;
	
	public static final int BTNEXECUTEVERIFICATION_HEIGHT = 20;
	
	// --- Passati dal modulo Main ---
			
	protected JComboBox cmbExamples;
	
	protected JComboBox cmbCircuits;
	
	// --- Nome e descrizione del circuito ---
	
	protected Label lblName;
	
	protected Label lblNameValue;
	
	protected Label lblDescription;
	
	protected Label lblDescriptionValue;
	
	protected Label lblInformations;
	
	protected JScrollPane scrlDescriptionValue;
	
	// --- Lista dei componenti, e lista di input e output ---
	
	protected JTable tblComponents;
	
	protected JTable tblInputs;
	
	protected JTable tblOutputs;
	
	protected JScrollPane scrlComponents;
	
	protected JScrollPane scrlInputs;
	
	protected JScrollPane scrlOutputs;
	
	// --- Lista di input e output del componente selezionato ---
	
	protected Label lblComponentInformations;
	
	protected Label lblComponentNote;
	
	protected JTable tblComponentInputs;
	
	protected JTable tblComponentOutputs;
	
	protected JScrollPane scrlComponentInputs;
	
	protected JScrollPane scrlComponentOutputs;
	
	// --- Verifica ---
	
	protected Label lblOptions;
	
	protected Label lblSpecification;
	
	protected JCheckBox chkBMC;
	
	protected Spinner spnBound;
	
	protected JTextArea txtSpecification;
	
	protected JScrollPane scrlSpecification;
	
	protected JButton btnExecuteVerification;
	
	protected JButton btnSCO;
	
	// --- Struttura dati ---
	
	protected HashMap< CircuitComponent, ArrayList< Boolean >[] > componentGates;
		
	// --- Parser, per poter risalire alle informazioni sul circuito ---
	
	protected VerilogParser parser;
	
	// --- Listener ---
	
	protected CircuitPanelListener circuitPanelListener;
	
	
	public CircuitPanel( int x, int y, JComboBox cmbExamples, JComboBox cmbCircuits )
	{
		super();
		
		this.cmbExamples = cmbExamples;
		
		this.cmbCircuits = cmbCircuits;
		
		setLayout( null );
		
		setSize( PANEL_WIDTH, PANEL_HEIGHT );
		
		setLocation( x, y );
		
		setBackground( Color.WHITE );
		
		initializeComponents();
	}
	
	public void loadCircuit( String fileName )
	{			
	
	//System.err.println("DEBUG: loadCircuit() - fileName: " + fileName);
		String descriptionFileName = fileName.substring( 0, fileName.length() - 3 ) + "dat";
		
		try
		{
			BufferedReader description = new BufferedReader( new FileReader( descriptionFileName ) );
		
		
			String line = description.readLine();
		
			if ( line != null )
		
				lblDescriptionValue.setText( "<HTML>" + line + "</HTML>" );
		
			else
				
				lblDescriptionValue.setText( "[ non disponibile ]" );
				
			
			line = description.readLine();
		
			if ( line != null )
		
				txtSpecification.setText( line );
		
			else
				
				txtSpecification.setText( "" );
				
			
			line = description.readLine();
		
			if ( line != null )
			{
				int bound = Integer.parseInt( line );
				
				if ( bound != -1 )
				{
					chkBMC.setSelected( true );
					
					spnBound.setEnabled( true );
					
					spnBound.setValue( bound );
				}				
					
				else
				{
					chkBMC.setSelected( false );
					
					spnBound.setEnabled( false );
				}						
			}			
		
			else
			{
				chkBMC.setSelected( false );
				
				spnBound.setEnabled( false );
			}				
		}
		
		catch ( Exception e )
		{
			lblDescriptionValue.setText( "[ non disponibile ]" );
			
			txtSpecification.setText( "" );
			
			chkBMC.setSelected( false );
		}
		
		
		lblNameValue.setText( fileName );
		
		
		parser = new VerilogParser( fileName );
		
		componentGates = new HashMap();
						
		// --- AGGIUNTA DEI COMPONENTI ---
		int componentCount = parser.getMainComponent().getComponentCount();
				
		String[][] cells = new String[ componentCount ][ 2 ];
						
		for ( int i = 0; i < componentCount; i++ )
		{
			CircuitComponent currentComponent = parser.getMainComponent().getComponent( i );
			
			ArrayList[] gates = new ArrayList[ 2 ];
			
			gates[ 0 ] = new ArrayList();
			
			gates[ 1 ] = new ArrayList();
			
			componentGates.put( currentComponent, gates );
			
			for ( int j = 0; j < currentComponent.getInputCount(); j++ )
				
				componentGates.get( currentComponent )[ 0 ].add( new Boolean( true ) );
			
			for ( int j = 0; j < currentComponent.getOutputCount(); j++ )
				
				componentGates.get( currentComponent )[ 1 ].add( new Boolean( true ) );
			
			cells[ i ][ 0 ] = parser.getMainComponent().getComponent( i ).getName();
				
			cells[ i ][ 1 ] = parser.getMainComponent().getComponent( i ).getType();	
		}			
		
		tblComponents = new JTable( cells, new String[] { "Nome componente", "Tipo componente" } );
		
		tblComponents.setSelectionMode( 0 );
				
		tblComponents.getSelectionModel().addListSelectionListener( new SelectionListener( this ) );
		
		tblComponents.setFont( lblName.getFont() );
		
		tblComponents.getTableHeader().setFont( lblName.getFont() );
		
		tblComponents.setRowSelectionInterval( 0, 0 );
		
		scrlComponents.setViewportView( tblComponents );
		
		scrlComponents.setBorder( null );
		// -------------------------------
	
		// --- AGGIUNTA DEGLI INPUTS ---
		int inputCount = parser.getMainComponent().getInputCount();
				
		cells = new String[ inputCount ][ 1 ];
						
		for ( int i = 0; i < inputCount; i++ )
		
			cells[ i ][ 0 ] = parser.getMainComponent().getInput( i );
					
		tblInputs = new JTable( cells, new String[] { "Input" } );
		
		tblInputs.setSelectionMode( 0 );
		
		tblInputs.setFont( lblName.getFont() );
		
		tblInputs.getTableHeader().setFont( lblName.getFont() );
		
		scrlInputs.setViewportView( tblInputs );
		
		scrlInputs.setBorder( null );
		// -----------------------------
		
		// --- AGGIUNTA DEGLI OUTPUTS ---
		int outputCount = parser.getMainComponent().getOutputCount();
				
		cells = new String[ outputCount ][ 1 ];
						
		for ( int i = 0; i < outputCount; i++ )
		
			cells[ i ][ 0 ] = parser.getMainComponent().getOutput( i );
					
		tblOutputs = new JTable( cells, new String[] { "Output" } );
		
		tblOutputs.setSelectionMode( 0 );
		
		tblOutputs.setFont( lblName.getFont() );
		
		tblOutputs.getTableHeader().setFont( lblName.getFont() );
		
		scrlOutputs.setViewportView( tblOutputs );
		
		scrlOutputs.setBorder( null );
		// ------------------------------
		
		
		lblInformations.setText( "Il circuito  costituito da " + componentCount + " componenti, e presenta " + inputCount + " inputs e " + outputCount + " outputs:" );
	}
	
	public void setSize( int width, int height )
	{
		// Dimensioni fisse.
		
		super.setSize( PANEL_WIDTH, PANEL_HEIGHT );
	}
	
	public void setCircuitPanelListener( CircuitPanelListener circuitPanelListener )
	{
		this.circuitPanelListener = circuitPanelListener;
	}
	
	protected void paintComponent( Graphics g )
	{
		super.paintComponent( g );
		
		g.setColor( new Color( 210, 210, 235 ) );
		
		g.drawLine( 0, 5 * LBLDESCRIPTION_HEIGHT + VSPACE / 2, PANEL_WIDTH, 5 * LBLDESCRIPTION_HEIGHT + VSPACE / 2 );
		
		g.drawLine( 2 * TBLCOMPONENTINPUTS_WIDTH + 3 * VSPACE / 2, 6 * LBLDESCRIPTION_HEIGHT + 5 * VSPACE / 2 + TBLCOMPONENTS_HEIGHT, PANEL_WIDTH, 6 * LBLDESCRIPTION_HEIGHT + 5 * VSPACE / 2 + TBLCOMPONENTS_HEIGHT );
		
		g.drawLine( 2 * TBLCOMPONENTINPUTS_WIDTH + 3 * VSPACE / 2, 6 * LBLDESCRIPTION_HEIGHT + 5 * VSPACE / 2 + TBLCOMPONENTS_HEIGHT, 2 * TBLCOMPONENTINPUTS_WIDTH + 3 * VSPACE / 2, PANEL_HEIGHT );
	}
	
	private void initializeComponents()
	{
		lblName = new Label( 0, 0, LBLDESCRIPTION_WIDTH, LBLDESCRIPTION_HEIGHT, "File:" );
		
		lblName.setFont( new Font( "Verdana", 0, 10 ) );
		
		add( lblName );
		
		
		lblNameValue = new Label( LBLDESCRIPTION_WIDTH, 0, PANEL_WIDTH - LBLDESCRIPTION_WIDTH, LBLDESCRIPTION_HEIGHT, "aval" );
	
		lblNameValue.setFont( lblName.getFont() );
	
		lblNameValue.setForeground( new Color( 100, 130, 255 ) );
	
		add( lblNameValue );
	
	
		lblDescription = new Label( 0, LBLDESCRIPTION_HEIGHT, LBLDESCRIPTION_WIDTH, LBLDESCRIPTION_HEIGHT, "Descrizione:" );
	
		lblDescription.setFont( lblName.getFont() );
		
		add( lblDescription );
		
		
		lblDescriptionValue = new Label( LBLDESCRIPTION_WIDTH - 15, LBLDESCRIPTION_HEIGHT, PANEL_WIDTH - LBLDESCRIPTION_WIDTH, 50 * LBLDESCRIPTION_HEIGHT, "val" );
	
		lblDescriptionValue.setFont( lblName.getFont() );
	
		lblDescriptionValue.setForeground( new Color( 80, 110, 180 ) );
	
		lblDescriptionValue.setVerticalAlignment( Label.TOP );
		
		scrlDescriptionValue = new JScrollPane();
		
		scrlDescriptionValue.setBounds( LBLDESCRIPTION_WIDTH, LBLDESCRIPTION_HEIGHT, PANEL_WIDTH - LBLDESCRIPTION_WIDTH, 4 * LBLDESCRIPTION_HEIGHT );
		
		scrlDescriptionValue.getViewport().setLayout( null );
		
		scrlDescriptionValue.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
		
		scrlDescriptionValue.getViewport().setBackground( Color.WHITE );
		
		scrlDescriptionValue.setBorder( null );
		
		scrlDescriptionValue.setViewportView( lblDescriptionValue );
		
		add( scrlDescriptionValue );
		
					
		lblInformations = new Label( 0, 5 * LBLDESCRIPTION_HEIGHT + VSPACE, PANEL_WIDTH, LBLDESCRIPTION_HEIGHT );
	
		lblInformations.setFont( lblName.getFont() );
		
		lblInformations.setForeground( lblNameValue.getForeground() );
		
		add( lblInformations );
					
						
		scrlComponents = new JScrollPane();
		
		scrlComponents.setBounds( 0, 5 * LBLDESCRIPTION_HEIGHT + 2 * VSPACE + LBLDESCRIPTION_HEIGHT, TBLCOMPONENTS_WIDTH, TBLCOMPONENTS_HEIGHT );
		
		scrlComponents.getViewport().setBackground( Color.WHITE );
										
		add( scrlComponents );
		
		
		scrlInputs = new JScrollPane();
		
		scrlInputs.setBounds( ( PANEL_WIDTH - TBLCOMPONENTS_WIDTH - 2 * TBLINPUTS_WIDTH ) / 2 + TBLCOMPONENTS_WIDTH, 5 * LBLDESCRIPTION_HEIGHT + 2 * VSPACE + LBLDESCRIPTION_HEIGHT, TBLINPUTS_WIDTH, TBLCOMPONENTS_HEIGHT );
					
		scrlInputs.getViewport().setBackground( Color.WHITE );
										
		add( scrlInputs );
		
		
		scrlOutputs = new JScrollPane();
		
		scrlOutputs.setBounds( ( PANEL_WIDTH - TBLCOMPONENTS_WIDTH - 2 * TBLINPUTS_WIDTH ) + TBLCOMPONENTS_WIDTH + TBLINPUTS_WIDTH, 5 * LBLDESCRIPTION_HEIGHT + 2 * VSPACE + LBLDESCRIPTION_HEIGHT, TBLINPUTS_WIDTH, TBLCOMPONENTS_HEIGHT );
		
		scrlOutputs.getViewport().setBackground( Color.WHITE );
							
		add( scrlOutputs );
		
		
		lblComponentInformations = new Label( 0, 5 * LBLDESCRIPTION_HEIGHT + 3 * VSPACE + LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT, 2 * TBLCOMPONENTINPUTS_WIDTH + VSPACE, 2 * LBLDESCRIPTION_HEIGHT );
		
		lblComponentInformations.setFont( lblName.getFont() );
		
		lblComponentInformations.setForeground( lblNameValue.getForeground() );
		
		add( lblComponentInformations );
		
		
		scrlComponentInputs = new JScrollPane();
		
		scrlComponentInputs.setBounds( 0, 5 * LBLDESCRIPTION_HEIGHT + 4 * VSPACE + 3 * LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT, TBLCOMPONENTINPUTS_WIDTH, TBLCOMPONENTS_HEIGHT );
		
		scrlComponentInputs.getViewport().setBackground( Color.WHITE );
							
		add( scrlComponentInputs );
		
		
		scrlComponentOutputs = new JScrollPane();
		
		scrlComponentOutputs.setBounds( TBLCOMPONENTINPUTS_WIDTH + VSPACE, 5 * LBLDESCRIPTION_HEIGHT + 4 * VSPACE + 3 * LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT, TBLCOMPONENTINPUTS_WIDTH, TBLCOMPONENTS_HEIGHT );
		
		scrlComponentOutputs.getViewport().setBackground( Color.WHITE );
							
		add( scrlComponentOutputs );
		
		
		lblComponentNote = new Label( 0, 5 * LBLDESCRIPTION_HEIGHT + 5 * VSPACE + 3 * LBLDESCRIPTION_HEIGHT + 2 * TBLCOMPONENTS_HEIGHT, 2 * TBLCOMPONENTINPUTS_WIDTH + VSPACE, 50, "<HTML><U>Nota</U>: solo gli input e gli output selezionati verranno mostrati nell'eventuale controesempio fornito da NuSMV.</HTML>" );
		
		lblComponentNote.setFont( lblName.getFont() );
		
		lblComponentNote.setForeground( lblDescriptionValue.getForeground() );
		
		add( lblComponentNote );
				
		
		lblOptions = new Label( 2 * TBLCOMPONENTINPUTS_WIDTH + 2 * VSPACE, 4 * LBLDESCRIPTION_HEIGHT + 4 * VSPACE + LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT, PANEL_WIDTH - 2 * TBLCOMPONENTINPUTS_WIDTH - 2 * VSPACE, 2 * LBLDESCRIPTION_HEIGHT, "Opzioni per la verifica automatica:" );
		
		lblOptions.setForeground( lblDescriptionValue.getForeground() );
		
		lblOptions.setFont( lblName.getFont() );
				
		add( lblOptions );
		
				
		chkBMC = new JCheckBox( "Bounded Model Checking, k = " );
		
		chkBMC.setBounds( 2 * TBLCOMPONENTINPUTS_WIDTH + 2 * VSPACE, 4 * LBLDESCRIPTION_HEIGHT + 5 * VSPACE + 3 * LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT, CHKBMC_WIDTH, LBLDESCRIPTION_HEIGHT );
		
		chkBMC.setFont( lblName.getFont() );
		
		chkBMC.setBackground( Color.WHITE );
		
		chkBMC.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed( ActionEvent e )
				{
					if ( chkBMC.isSelected() )
						
						spnBound.setEnabled( true );
						
					else
						
						spnBound.setEnabled( false );
				}
			}
		);
		
		add( chkBMC );
		
		
		spnBound = new Spinner( chkBMC.getLocation().x + chkBMC.getSize().width + SPNDISTANCE, 4 * LBLDESCRIPTION_HEIGHT + 5 * VSPACE + 3 * LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT - ( Spinner.HEIGHT - LBLDESCRIPTION_HEIGHT ) / 2, 10 );
		
		spnBound.setEnabled( false );
		
		add( spnBound );
		
		
		lblSpecification = new Label( 2 * TBLCOMPONENTINPUTS_WIDTH + 2 * VSPACE, 4 * LBLDESCRIPTION_HEIGHT + 6 * VSPACE + 4 * LBLDESCRIPTION_HEIGHT + TBLCOMPONENTS_HEIGHT, PANEL_WIDTH - 2 * TBLCOMPONENTINPUTS_WIDTH - 2 * VSPACE, LBLDESCRIPTION_HEIGHT, "Specifica LTL da verificare: " );
		
		lblSpecification.setFont( lblName.getFont() );
				
		add( lblSpecification );
		
		
		txtSpecification = new JTextArea();
				
		txtSpecification.setFont( new Font( "Courier New", 0, 12 ) );
		
		txtSpecification.setLineWrap( true );
		
		scrlSpecification = new JScrollPane( txtSpecification );
		
		scrlSpecification.setBounds( lblSpecification.getLocation().x, lblSpecification.getLocation().y + LBLDESCRIPTION_HEIGHT + VSPACE, lblSpecification.getSize().width, TXTSPECIFICATION_HEIGHT );
				
		add( scrlSpecification );
		
		
		btnExecuteVerification = new JButton( "Esegui verifica automatica" );
		
		btnExecuteVerification.setBounds( scrlSpecification.getLocation().x + VSPACE / 2 + 9 * ( scrlSpecification.getSize().width - VSPACE / 2 ) / 32, scrlSpecification.getLocation().y + TXTSPECIFICATION_HEIGHT + VSPACE, 23 * ( scrlSpecification.getSize().width - VSPACE / 2 ) / 32, BTNEXECUTEVERIFICATION_HEIGHT );
		
		btnExecuteVerification.setFont( lblName.getFont() );
		
		btnExecuteVerification.setBackground( new Color( 240, 240, 255 ) );
		
		btnExecuteVerification.setForeground( lblNameValue.getForeground() );
		
		btnExecuteVerification.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed( ActionEvent e )
				{
					executeVerification();
				}					
			}
		);
		
		add( btnExecuteVerification );
		
		
		btnSCO = new JButton( "SCO..." );
		
		btnSCO.setBounds( scrlSpecification.getLocation().x, scrlSpecification.getLocation().y + TXTSPECIFICATION_HEIGHT + VSPACE, 9 * ( scrlSpecification.getSize().width - VSPACE / 2 ) / 32, BTNEXECUTEVERIFICATION_HEIGHT );
		
		btnSCO.setFont( lblName.getFont() );
		
		btnSCO.setBackground( new Color( 240, 240, 255 ) );
		
		btnSCO.setForeground( lblNameValue.getForeground() );
		
		add( btnSCO );
	}

/**********	
	private void executeVerification_BAK()
	{	
		if ( circuitPanelListener != null )
			
			circuitPanelListener.verificationStarted();
			
		try
		{
			// --- Disabilitazione dei componenti ---
				
			btnExecuteVerification.setEnabled( false );
			
			btnSCO.setEnabled( false );
			
			txtSpecification.setEnabled( false );
			
			chkBMC.setEnabled( false );
			
			spnBound.setEnabled( false );
			
			tblComponents.setEnabled( false );
			
			tblInputs.setEnabled( false );
			
			tblOutputs.setEnabled( false );
			
			tblComponentInputs.setEnabled( false );
			
			tblComponentOutputs.setEnabled( false );
			
			// --------------------------------------
			
			
			PrintWriter fileWriter = new PrintWriter( new File( "../RunNuSMV.bat" ) );
			
			fileWriter.println( "cd.." );
			
			fileWriter.println( "cd.." );
			
			fileWriter.println( "cd NuSMV/2.4.3/bin" );
			
			
			File sourceVerilogFile = new File( lblNameValue.getText() );
						
			String verilogPath = sourceVerilogFile.getAbsolutePath();
			
			String smvPath = verilogPath.substring( 0, verilogPath.length() - 3 ) + "smv";
									
			new VerilogToSMVConverter( verilogPath, smvPath, new LTLSpecification( txtSpecification.getText() ) );
			
			
			String outPath = verilogPath.substring( 0, verilogPath.length() - 3 ) + "out";
			
			( new File( outPath ) ).delete();
						
			smvPath = "../../../Tesina/Codice/" + lblNameValue.getText().substring( 3, lblNameValue.getText().length() - 3 ) + "smv";
			
			outPath = "../../../Tesina/Codice/" + lblNameValue.getText().substring( 3, lblNameValue.getText().length() - 3 ) + "out";
						
			fileWriter.println( "NuSMV " + ( ( chkBMC.isSelected() ) ? ( "-bmc -bmc_length " + spnBound.getValue() + " " ) : ( "" ) ) + "\"" + smvPath + "\" > \"" + outPath + "\"" );
			
			
			fileWriter.println( "cd.." );
			
			fileWriter.println( "cd.." );
			
			fileWriter.println( "cd.." );
			
			fileWriter.println( "cd Tesina/Codice" );
			
			fileWriter.close();
			
			
			outPath = verilogPath.substring( 0, verilogPath.length() - 3 ) + "out";
			
			new OutputFrame( this, outPath, parser, componentGates );
		}
		
		catch ( Exception e )
		{
			System.out.println( e.getStackTrace() );
		}	
	}
*******/	
	private void executeVerification()  /// TM: QUELLO VERO...................................
	{	
		if ( circuitPanelListener != null )
			
			circuitPanelListener.verificationStarted();
			
		try
		{
			// --- Disabilitazione dei componenti ---
				
			btnExecuteVerification.setEnabled( false );
			
			btnSCO.setEnabled( false );
			
			txtSpecification.setEnabled( false );
			
			chkBMC.setEnabled( false );
			
			spnBound.setEnabled( false );
			
			tblComponents.setEnabled( false );
			
			tblInputs.setEnabled( false );
			
			tblOutputs.setEnabled( false );
			
			tblComponentInputs.setEnabled( false );
			
			tblComponentOutputs.setEnabled( false );
			
			// --------------------------------------
			
			/*
			PrintWriter fileWriter = new PrintWriter( new File( "../RunNuSMV.bat" ) );
			*/			

			File sourceVerilogFile = new File( lblNameValue.getText() );
						
			String verilogPath = sourceVerilogFile.getAbsolutePath();
			
	//System.err.println("verilogPath: " + verilogPath);		
			
			
			String smvPath = verilogPath.substring( 0, verilogPath.lastIndexOf('.') ) + ".smv"; //TM  verilogPath.length() - 3 ) + "smv";

	//System.err.println("verilogPath: " + verilogPath);
										
			new VerilogToSMVConverter( verilogPath, smvPath, new LTLSpecification( txtSpecification.getText() ) );
			
			
			String outPath = verilogPath.substring( 0, verilogPath.lastIndexOf('.') ) + ".out";
	//System.err.println("verilogPath: " + verilogPath);	
					
			( new File( outPath ) ).delete();
						
			smvPath = lblNameValue.getText().substring( 0, lblNameValue.getText().lastIndexOf('.') ) + ".smv";
			
			outPath = lblNameValue.getText().substring( 0, lblNameValue.getText().lastIndexOf('.') ) + ".out";
						
			//TM fileWriter.println( "NuSMV " + ( ( chkBMC.isSelected() ) ? ( "-bmc -bmc_length " + spnBound.getValue() + " " ) : ( "" ) ) + "\"" + smvPath + "\" > \"" + outPath + "\"" );
			// TM added
			String command = "NuSMV " + ( ( chkBMC.isSelected() ) ? ( "-bmc -bmc_length " + spnBound.getValue() + " " ) : ( "" ) ) + "\"" + smvPath + "\" > \"" + outPath + "\"";
			
			//TM fileWriter.close();

			//System.err.println("DEBUG>>>>> lblNameValue: " + lblNameValue.getText()); //TM
			//System.err.println("DEBUG>>>>> command: " + command); //TM
			
			outPath = verilogPath.substring( 0, verilogPath.lastIndexOf('.') ) + ".out";
			
			new OutputFrame( this, outPath, parser, componentGates, command );
		}
		
		catch ( Exception e )
		{
			System.out.println( e.getStackTrace() );
		}	
	}
}

class SelectionListener implements ListSelectionListener
{
	private CircuitPanel pnlCircuit;
	
	
	public SelectionListener( CircuitPanel pnlCircuit )
	{
		this.pnlCircuit = pnlCircuit;
	}
	
	public void valueChanged( ListSelectionEvent e )
	{
		int selectedComponentIndex = pnlCircuit.tblComponents.getSelectedRow();
		
		if ( selectedComponentIndex != -1 )
		{
			CircuitComponent currentComponent = pnlCircuit.parser.getMainComponent().getComponent( selectedComponentIndex );
			
			ArrayList[] gates = pnlCircuit.componentGates.get( currentComponent );
			
			
			int inputCount = currentComponent.getInputCount();
				
			Object[][] cells = new Object[ inputCount ][ 2 ];
							
			for ( int i = 0; i < inputCount; i++ )
			{			
				cells[ i ][ 0 ] = currentComponent.getInput( i );
			
				cells[ i ][ 1 ] = gates[ 0 ].get( i );
			}
			
			pnlCircuit.tblComponentInputs = new JTable( new TableModel( cells, new String[] { "Input", "" } ) );
			
			pnlCircuit.tblComponentInputs.setSelectionMode( 0 );
			
			pnlCircuit.tblComponentInputs.getModel().addTableModelListener
			(
				new TableModelListener()
				{
					public void tableChanged( TableModelEvent e )
					{
						CircuitComponent selectedComponent = pnlCircuit.parser.getMainComponent().getComponent( pnlCircuit.tblComponents.getSelectedRow() );
						
						ArrayList[] gates = pnlCircuit.componentGates.get( selectedComponent );
						
						for ( int i = 0; i < gates[ 0 ].size(); i++ )
						{
							gates[ 0 ].remove( i );
							
							gates[ 0 ].add( i, pnlCircuit.tblComponentInputs.getValueAt( i, 1 ) );
						}						
					}
				}
			);
			
			pnlCircuit.tblComponentInputs.getTableHeader().addMouseListener
			(
				new MouseAdapter()
				{
					public void mouseClicked( MouseEvent e )
					{
						if ( !pnlCircuit.tblComponentInputs.isEnabled() )
							
							return;
						
						for ( int i = 0; i < pnlCircuit.tblComponentInputs.getRowCount(); i++ )
						{
							if ( e.getButton() == MouseEvent.BUTTON1 )
							
								pnlCircuit.tblComponentInputs.setValueAt( new Boolean( true ), i, 1 );
						
							else if ( e.getButton() == MouseEvent.BUTTON3 )
								
								pnlCircuit.tblComponentInputs.setValueAt( new Boolean( false ), i, 1 );
						}
					}						
				}
			);
						
			pnlCircuit.tblComponentInputs.setFont( pnlCircuit.lblName.getFont() );
			
			pnlCircuit.tblComponentInputs.getTableHeader().setFont( pnlCircuit.lblName.getFont() );
			
			pnlCircuit.scrlComponentInputs.setViewportView( pnlCircuit.tblComponentInputs );
			
			pnlCircuit.scrlComponentInputs.setBorder( null );	
				
			
			int outputCount = pnlCircuit.parser.getMainComponent().getComponent( selectedComponentIndex ).getOutputCount();
				
			cells = new Object[ outputCount ][ 2 ];
							
			for ( int i = 0; i < outputCount; i++ )
			{
				cells[ i ][ 0 ] = pnlCircuit.parser.getMainComponent().getComponent( selectedComponentIndex ).getOutput( i );
			
				cells[ i ][ 1 ] = gates[ 1 ].get( i );
			}
						
			pnlCircuit.tblComponentOutputs = new JTable( new TableModel( cells, new String[] { "Output", "" } ) );
			
			pnlCircuit.tblComponentOutputs.setSelectionMode( 0 );
			
			pnlCircuit.tblComponentOutputs.getModel().addTableModelListener
			(
				new TableModelListener()
				{
					public void tableChanged( TableModelEvent e )
					{
						CircuitComponent selectedComponent = pnlCircuit.parser.getMainComponent().getComponent( pnlCircuit.tblComponents.getSelectedRow() );
						
						ArrayList[] gates = pnlCircuit.componentGates.get( selectedComponent );
						
						for ( int i = 0; i < gates[ 1 ].size(); i++ )
						{
							gates[ 1 ].remove( i );
							
							gates[ 1 ].add( i, pnlCircuit.tblComponentOutputs.getValueAt( i, 1 ) );
						}						
					}
				}
			);
			
			pnlCircuit.tblComponentOutputs.getTableHeader().addMouseListener
			(
				new MouseAdapter()
				{
					public void mouseClicked( MouseEvent e )
					{
						if ( !pnlCircuit.tblComponentOutputs.isEnabled() )
							
							return;
						
						for ( int i = 0; i < pnlCircuit.tblComponentOutputs.getRowCount(); i++ )
						{
							if ( e.getButton() == MouseEvent.BUTTON1 )
							
								pnlCircuit.tblComponentOutputs.setValueAt( new Boolean( true ), i, 1 );
						
							else if ( e.getButton() == MouseEvent.BUTTON3 )
								
								pnlCircuit.tblComponentOutputs.setValueAt( new Boolean( false ), i, 1 );
						}
					}						
				}
			);
			
			pnlCircuit.tblComponentOutputs.setFont( pnlCircuit.lblName.getFont() );
			
			pnlCircuit.tblComponentOutputs.getTableHeader().setFont( pnlCircuit.lblName.getFont() );
			
			pnlCircuit.scrlComponentOutputs.setViewportView( pnlCircuit.tblComponentOutputs );
			
			pnlCircuit.scrlComponentOutputs.setBorder( null );
			
			
			pnlCircuit.lblComponentInformations.setText( "<HTML>Il componente selezionato presenta " + inputCount + " inputs e " + outputCount + " outputs:</HTML>" );				
		}
		
		else
		{
			pnlCircuit.scrlComponentInputs.setViewportView( null );
			
			pnlCircuit.scrlComponentInputs.setBorder( null );
			
			pnlCircuit.scrlComponentOutputs.setViewportView( null );
			
			pnlCircuit.scrlComponentOutputs.setBorder( null );
			
			pnlCircuit.lblComponentInformations.setText( "<HTML>Nessun componente selezionato</HTML>" );
		}
	}
}

class TableModel extends AbstractTableModel
{
	private Object[][] data;
	
	private String[] columnNames;
         
    
    public TableModel( Object[][] data, String[] columnNames )
    {
    	this.data = data;
    	
    	this.columnNames = columnNames;
    }
    
    public int getColumnCount()
	{
        return columnNames.length;
    }

    public int getRowCount()
    {
        return data.length;
    }

    public String getColumnName( int column )
    {
        return columnNames[ column ];
    }

    public Object getValueAt( int row, int column )
    {
        return data[ row ][ column ];
    }
	
	public Class getColumnClass( int column )
	{        				
		return getValueAt( 0, column ).getClass();
	}
	
	public boolean isCellEditable( int row, int column )
	{        
        if ( column < 1 )
        
            return false;
        
         else
            
            return true;
	}
	
	public void setValueAt( Object value, int row, int column )
	{
        data[ row ][ column ] = value;
        
        fireTableCellUpdated( row, column );
    }

}