package gui;

import circuit.CircuitComponent;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;


import parser.VerilogParser;


public class OutputFrame extends JFrame
{
	public static final int CLIENT_WIDTH = 450;
	
	public static final int CLIENT_HEIGHT = 500;
	
	public static final int BORDER = 15;
	
	public static final int LBLINFORMATIONS_HEIGHT = 85; 
	
	
	private CircuitPanel pnlCircuit;
	
	private String outputFileName;
	
	private VerilogParser parser;
	
	private HashMap< CircuitComponent, ArrayList< Boolean >[] > componentGates;
	
	private Label lblInformations;
	
	private JTextField commandTF; // TM
	
	private JTextArea txtOutput;
	
	private JScrollPane scrlOutput;
	
	private Thread loadingThread;
	
	private boolean exit;
	
		
	public OutputFrame( CircuitPanel pnlCircuit, String outputFileName, VerilogParser parser, HashMap componentGates, String command )
    {
		/*RuntimeException e = new RuntimeException("By Toni");
		if (1==1) {
			e.printStackTrace();
		}*/
			
    	this.pnlCircuit = pnlCircuit;
    	
    	this.outputFileName = outputFileName;    	
    	
    	this.parser = parser;
    	
    	this.componentGates = componentGates;
    	
    	initializeWindow();
    	   
    	initializeGUIComponents(command);
    	
    	exit = false;
    	
    	final String fixedOutputFileName = outputFileName;
    	
    	loadingThread = new Thread()
    	{
    		public void run()
    		{    				
    			while ( !exit )
    			
    				loadOutputFile();
    		}
    	};
    	
    	loadingThread.start();
    	     	
    	repaint();
    }
    
    private void loadOutputFile()
    {
    	String output = "";
    	    	    	
    	try
    	{
    		// Attesa della creazione del file...
    		
    		File outputFile = new File( outputFileName );
    		
    		while ( !outputFile.exists() )
    			
    			outputFile = new File( outputFileName );
    			
    		    		    		
    		FileInputStream inputStream = new FileInputStream( outputFile );
    		
    		InputStreamReader inputStreamReader = new InputStreamReader( inputStream );
    		
    		BufferedReader bufferedReader = new BufferedReader( inputStreamReader );
    		
    		
    		String currentLine = bufferedReader.readLine();
    		    		
    		while ( currentLine != null )
    		{
    			output += ( acceptLine( currentLine ) );
    			
    			currentLine = bufferedReader.readLine();
    		}
    		
    		if ( !output.equals( txtOutput.getText() ) )
    			
    			txtOutput.setText( output );
    			
    			
    		bufferedReader.close();
    		
    		inputStreamReader.close();
    		
    		inputStream.close();
    	}
    	
    	catch ( Exception e )
    	{
    		e.printStackTrace();
    	} 
    }
    
    private String acceptLine( String line )
    {
    	StringTokenizer tokenizer = new StringTokenizer( line.trim(), " \t\n\r\f." );
    	
    	String firstToken;
    	
    	if ( tokenizer.hasMoreTokens() )
    	
    		firstToken = tokenizer.nextToken();
    	   		
    	else
    		
    		return line + "\n";
    	
    
    	boolean isComponent = false;
    	
    	CircuitComponent mainComponent = parser.getMainComponent();
    	
    	CircuitComponent foundComponent = null;
    	
    	for ( int i = 0; i < mainComponent.getComponentCount(); i++ )
    	{    	
    		if ( firstToken.equals( mainComponent.getComponent( i ).getName() ) & !isComponent )
    		{
    			isComponent = true;
    			
    			foundComponent = mainComponent.getComponent( i );
    		}
    	}
    	
	    if ( isComponent )
	    {
	    	String processedLine;
	    	
	    	String secondToken = tokenizer.nextToken();
	    	
	    	ArrayList[] gates = componentGates.get( foundComponent );
	    		    		
	    		    		
	    	boolean isInput = false;
	    		
	    	for ( int j = 0; j < foundComponent.getInputCount(); j++ )
	    	
	    		if ( secondToken.equals( "INPUTS" ) || secondToken.equals( "REVERSE_INPUTS" ) || ( secondToken.equals( foundComponent.getInput( j ) ) & ( ( Boolean )( gates[ 0 ].get( j ) ) ).booleanValue() ) )
	    		{
	    			isInput = true;
	    			
	    			break;
	    		}
	    		
	    	boolean isOutput = false;
	    		
	    	for ( int j = 0; j < foundComponent.getOutputCount(); j++ )
	    	
	    		if ( secondToken.equals( "OUTPUTS" ) || secondToken.equals( "REVERSE_OUTPUTS" ) || ( secondToken.equals( foundComponent.getOutput( j ) ) & ( ( Boolean )( gates[ 1 ].get( j ) ) ).booleanValue() ) )
	    		{
	    			isOutput = true;
	    			
	    			break;
	    		}
	    			
	    	if ( isInput || isOutput )
	    	{
	    		tokenizer.nextToken();
	    		
	    		processedLine = firstToken + "." + secondToken + " = " + decimalToBits( tokenizer.nextToken() ) + "\n";
	    			
	    		return processedLine;
	    	}
	    			
	    	else
	    			
	    		return "";
	    }
    	    	
    	else
    		
    		return line + "\n";
    }
    
    private String decimalToBits( String decimalExpression )
    {
    	if ( !decimalExpression.startsWith( "0d" ) )
    		
    		return decimalExpression;
    	
    	int underscoreIndex = decimalExpression.indexOf( '_' );
    	
    	int bitCount = Integer.parseInt( decimalExpression.substring( 2, underscoreIndex ) );
    	
    	int decimal = Integer.parseInt( decimalExpression.substring( underscoreIndex + 1 ) );
    	
    	ArrayList bits = new ArrayList();
    	
    	for ( int i = 0; i < bitCount; i++ )
    		
    		bits.add( new Boolean( false ) );
    	
    	int bitIndex = 0;
    	
    	while( decimal > 0 )
    	{
    		bits.remove( bitIndex );
    		
    		bits.add( bitIndex, ( decimal % 2 ) == 1 );
    			
    		decimal /= 2;
    		
    		bitIndex++;
    	}
    	
    	String output = "0b" + bitCount + "_";
    	
    	for ( int i = bits.size() - 1; i >= 0; i-- )
    	{
    		boolean bit = ( ( Boolean )( bits.get( i ) ) ).booleanValue();
    		
    		output += ( ( bit ) ? ( "1" ) : ( "0" ) );
    	}	
    		     	    		
    	return output;
    }
    
    private void initializeWindow()
    {
    	setTitle( "Verifica formale di circuiti digitali - Output" );
    	
    	setResizable( false );
    	
    	setVisible( true );
    	
    	setSize( CLIENT_WIDTH + getInsets().left + getInsets().right, CLIENT_HEIGHT + getInsets().top + getInsets().bottom );
    	
    	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    	
    	setLocation( ( screen.width - getSize().width ) / 2, ( screen.height - getSize().height ) / 2 );
        
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        
    	getContentPane().setBackground( Color.WHITE );
    	
    	addWindowListener
    	(
    		new WindowAdapter()
    		{
    			public void windowClosing( WindowEvent e )
    			{   				
    				exit = true;
    				
    				if ( pnlCircuit.circuitPanelListener != null )
    					
    					pnlCircuit.circuitPanelListener.verificationFinished();
    				
    				pnlCircuit.btnExecuteVerification.setEnabled( true );
			
					pnlCircuit.btnSCO.setEnabled( true );
					
					pnlCircuit.txtSpecification.setEnabled( true );
					
					pnlCircuit.chkBMC.setEnabled( true );
					
					pnlCircuit.spnBound.setEnabled( true );
					
					pnlCircuit.tblComponents.setEnabled( true );
					
					pnlCircuit.tblInputs.setEnabled( true );
					
					pnlCircuit.tblOutputs.setEnabled( true );
					
					pnlCircuit.tblComponentInputs.setEnabled( true );
					
					pnlCircuit.tblComponentOutputs.setEnabled( true );
    			}
    		}
    	);
    	    	
    	setLayout( null );    	
    }
    
    private void initializeGUIComponents(String command)
    {
    	lblInformations = new Label( BORDER, BORDER, CLIENT_WIDTH - 2 * BORDER, LBLINFORMATIONS_HEIGHT );
    	
    	lblInformations.setText( "<HTML>Si prega di far partire manualmente il file RunNuSMV.bat: non appena l'output prodotto da NuSMV sar disponibile, verr visualizzato nel riquadro sottostante. Qualora la computazione richiedesse troppo tempo, sar possibile chiudere tranquillamente questa finestra (in tal caso, per, ci si ricordi di forzare la terminazione di NuSMV, utilizzando il Task Manager).</HTML>" );
    	
    	lblInformations.setForeground( new Color( 80, 110, 180 ) );
    	
    	lblInformations.setFont( new Font( "Verdana", 0, 10 ) );
    	
    	add( lblInformations );
    	
    	
		commandTF = new JTextField();
		
		commandTF.setFont( new Font( "Courier New", 0, 11 ) );
    	
    	commandTF.setBounds( BORDER, 2 * BORDER + LBLINFORMATIONS_HEIGHT, lblInformations.getSize().width, 15 );
	    	
		commandTF.setText(command);
    	add( commandTF );

		
		
    	txtOutput = new JTextArea();
    	    	    	
    	txtOutput.setFont( new Font( "Courier New", 0, 11 ) );
    	
    	scrlOutput = new JScrollPane( txtOutput );
    	
    	scrlOutput.setBounds( BORDER, 2 * BORDER + LBLINFORMATIONS_HEIGHT + 20, lblInformations.getSize().width, CLIENT_HEIGHT - 3 * BORDER - LBLINFORMATIONS_HEIGHT - 20);
    	    	
    	add( scrlOutput );
    }
}