package gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;


public class Spinner extends JPanel
{
	public static final int WIDTH = 48;
	
	public static final int HEIGHT = 17;
	
	
	private int value;
	
	private int state;
	
	private SpinnerListener listener;
	
		
	public Spinner( int x, int y, int value )
	{
		this.value = value;
		
		state = 0;
		
		
		setBounds( x, y, WIDTH, HEIGHT );
	
		setBackground( new Color( 255, 255, 255 ) );
		
		setFont( new Font( "Verdana", 0, 10 ) );
		
		
		setMouseManagers();
	}
	
	public int getValue()
	{
		return value;
	}

	public void setValue( int value )
	{
		this.value = value;
		
		repaint();
	}
	

	public void setSpinnerListener( SpinnerListener listener )
	{
		this.listener = listener;
	}
		
	
	private void setMouseManagers()
	{
		addMouseListener
		(
			new MouseAdapter()
			{
				public void mouseClicked( MouseEvent e )
				{
					if ( !isEnabled() )
						
						return;
					
					if ( state == 0 )
					
						return;
						
					
					if ( ( state == 1 ) && ( value < 99 ) )
					
						value++;
						
					else if ( ( state == 2 ) && ( value >= 1 ) )
					
						value--;
				
					
					if ( listener != null )
				
						listener.valueChanged( value );
						
				
					repaint();
				}
				
				
				public void mouseExited( MouseEvent e )
				{
					if ( !isEnabled() )
						
						return;
					
					state = 0;
					
					repaint();
				}				
			}
		);
		
		
		addMouseMotionListener
		(
			new MouseMotionAdapter()
			{
				public void mouseMoved( MouseEvent e )
				{
					if ( !isEnabled() )
						
						return;
					
					if ( e.getX() >= 35 )
					{
						if ( e.getY() <= 8 )
						
							state = 1;
							
						else
						
							state = 2;
					}
					
					else
						
						state = 0;						
				
					repaint();
				}				
			}
		);
	}
	
	
	protected void paintComponent( Graphics g )
	{
		super.paintComponent( g );
		
		if ( !isEnabled() )
			
			g.setColor( new Color( 210, 210, 235 ) );
						
		else
			
			g.setColor( new Color( 100, 130, 255 ) );
		
		
		g.drawRect( 0, 0, WIDTH - 1, HEIGHT - 1 );		
		
		g.drawLine( 34, 1, 34, 15 );		
		
		g.drawLine( 35, 8, 46, 8 );
		
		
		if ( state == 1 )
			
			if ( !isEnabled() )
			
				g.setColor( new Color( 210, 210, 235 ) );
						
			else
					
				g.setColor( new Color( 180, 210, 230 ) );
			
				
		g.drawLine( 36, 6, 45, 6 );
		
		g.drawLine( 37, 5, 44, 5 );
		
		g.drawLine( 38, 4, 43, 4 );
		
		g.drawLine( 39, 3, 42, 3 );
		
		g.drawLine( 40, 2, 41, 2 );
		
		
		if ( state == 2 )
		{
			if ( !isEnabled() )
			
				g.setColor( new Color( 210, 210, 235 ) );
						
			else
				
				g.setColor( new Color( 180, 210, 230 ) );
		}
		
		else
			
			if ( !isEnabled() )
			
				g.setColor( new Color( 210, 210, 235 ) );
						
			else
				
				g.setColor( new Color( 100, 130, 255 ) );
			
					
		g.drawLine( 36, 10, 45, 10 );
		
		g.drawLine( 37, 11, 44, 11 );
		
		g.drawLine( 38, 12, 43, 12 );
		
		g.drawLine( 39, 13, 42, 13 );
		
		g.drawLine( 40, 14, 41, 14 );
		
		
		if ( !isEnabled() )
			
			g.setColor( new Color( 210, 210, 235 ) );
						
		else
			
			g.setColor( new Color( 100, 130, 255 ) );
		
		
		int textX = ( 37 - getFontMetrics( getFont() ).stringWidth( "" + value ) ) / 2 + 1;
		
		int textY = ( 15 - getFontMetrics( getFont() ).getHeight() ) / 2 - 3 + getFontMetrics( getFont() ).getHeight();
		
		g.setFont( getFont() );
		
		g.drawString( "" + value, textX, textY );
	}
}