/*
 * localDir
 *
 * Utility class to get String and File reference to the local directory
 * that the class is executing in.
 */

package ldir;

import java.io.File;
/**
 *
 * @author  Mark Kozel
 * @version 1.4.1
 * @since Created on December 16, 2002
 */
public final class localDir
{

   private localDir()
   {
   }
   
   /**
    * Returns the disk file name of the class that is executing.
    *
    * @param none
    * @return Name of class that is currently executing
    */   
   public static String getClassName(Class c)
   {
      String thisClassName;
      
      //Build a string with executing class's name
      thisClassName = c.getName();
      thisClassName = thisClassName.substring(thisClassName.lastIndexOf(".") + 1,thisClassName.length());
      thisClassName += ".class";  //this is the name of the bytecode file that is executing
      
      return thisClassName;
   }
   
   /**
    * Returns the name of the local directory based on the results of a call to getClassName()
    *
    * @param none
    * @return Name of directory that contains the executing class
    */      
   public static String getLocalDirName(Class c)
   {
      String localDirName;
      
      //Use that name to get a URL to the directory we are executing in
      java.net.URL myURL = c.getResource(getClassName(c));  //Open a URL to the our .class file
      
      //Clean up the URL and make a String with absolute path name
      localDirName = myURL.getPath();  //Strip path to URL object out
      localDirName = myURL.getPath().replaceAll("%20", " ");  //change %20 chars to spaces  
      
      //Get the current execution directory
      localDirName = localDirName.substring(0,localDirName.lastIndexOf("/"));  //clean off the file name
      
      return localDirName;
   }

   /**
    * Returns a File reference to the local directory based on the results of a call to getClassName()
    *
    * @param none
    * @return File object that points to the local directory
    */      
   public static java.io.File getLocalDirRef(Class c)
   {
      File myFileObj;
      //Make the file object and return it
      myFileObj = new File(getLocalDirName(c));
      return myFileObj;
   }

}
