package parser;

import circuit.CircuitComponent;

import java.io.FileNotFoundException;
import java.io.PrintWriter;

import java.util.ArrayList;

public class SMVBuilder
{
	private ArrayList<SMVModule> modules;
	
	private LTLSpecification specification;
	
	
	public SMVBuilder()
	{
		modules = new ArrayList();
	
		specification = null;
	}
	
	public void addModule( SMVModule module )
	{
		// Aggiungiamo un modulo alla lista solo se esso non  gi presente.
		// Se non facessimo cos, potremmo ottenere dei moduli duplicati.
		
		if ( modules.indexOf( module ) == -1 )
			
			modules.add( module );
	}
				
	public void setLTLSpecification( LTLSpecification specification )
	{
		this.specification = specification;
	}
	
	public void saveToFile( String fileName ) throws FileNotFoundException
	{
		PrintWriter toFile = new PrintWriter( fileName );
		
		for ( int i = 0; i < modules.size(); i++ )
		{	
			SMVModule currentModule = modules.get( i );
			
			// --- BLOCCO INIZIALE, CON IL NOME DEL MODULO ---			
			toFile.println( "MODULE " + currentModule.getName() );
			
			toFile.println();			
			// -----------------------------------------------			
			
			// --- BLOCCO DELLE VARIABILI ---	
			if ( currentModule.getVariableCount() > 0 )
			{
				toFile.println( "\tVAR" );		
				
				for ( int j = 0; j < currentModule.getVariableCount(); j++ )
				{
					SMVVariable currentVariable = currentModule.getVariable( j );
					
					toFile.println( "\t\t" + currentVariable.getName() + "\t:\t" + currentVariable.getType() + ";" );
				}					
				
				toFile.println();
			}
			// ------------------------------
			
			// --- BLOCCO DELLE ASSEGNAZIONI ---
			if ( currentModule.getAssignmentCount() > 0 )
			{
				toFile.println( "\tASSIGN" );		
				
				for ( int j = 0; j < currentModule.getAssignmentCount(); j++ )
				{
					SMVAssignment currentAssignment = currentModule.getAssignment( j );
					
					toFile.println( "\t\t" + currentAssignment.getLeftSide() + " := " + currentAssignment.getRightSide() + ";" );
				}					
				
				toFile.println();
			}
			// ---------------------------------

			// --- BLOCCO DELLE DEFINIZIONI ---
			if ( currentModule.getDefinitionCount() > 0 )
			{
				toFile.println( "\tDEFINE" );		
				
				for ( int j = 0; j < currentModule.getDefinitionCount(); j++ )
				{
					SMVDefinition currentDefinition = currentModule.getDefinition( j );
					
					toFile.println( "\t\t" + currentDefinition.getDefinedVariable() + "\t:=\t" + currentDefinition.getDefinition() + ";" );
				}					
				
				toFile.println();
			}
			// --------------------------------
			
			// --- BLOCCO DEGLI INVARIANTI ---
			if ( currentModule.getInvariantCount() > 0 )
			{
				toFile.println( "\tINVAR" );		
				
				for ( int j = 0; j < currentModule.getInvariantCount(); j++ )
				{
					SMVInvariant currentInvariant = currentModule.getInvariant( j );
					
					toFile.println( "\t\t( " + currentInvariant.getInvariant() + " )" + ( ( j == currentModule.getInvariantCount() - 1 ) ? ( "" ) : ( " &" ) ) );
				}					
				
				toFile.println();
			}
			// -------------------------------
			
			// --- BLOCCO DELLE TRANSIZIONI ---
			if ( currentModule.getTransitionCount() > 0 )
			{
				toFile.println( "\tTRANS" );		
				
				toFile.println( "\t\tcase" );
				
				for ( int j = 0; j < currentModule.getTransitionCount(); j++ )
				{
					SMVTransition currentTransition = currentModule.getTransition( j );
					
					toFile.println( "\t\t\t" + currentTransition.getBeforeSide() + "\t:\t" + currentTransition.getAfterSide() + ";" );
				}					
				
				toFile.println( "\t\tesac" );
				
				toFile.println();
			}
			// --------------------------------									
		}	
		
		toFile.println( "LTLSPEC" );
			
		toFile.println();
			
		toFile.print( specification.getSpecification() );
		
		toFile.close();
	}
}