package parser;

import java.util.ArrayList;

public class SMVModule
{
	private String name;
	
	private ArrayList<SMVVariable> variables;
	
	private ArrayList<SMVAssignment> assignments;
	
	private ArrayList<SMVDefinition> definitions;
	
	private ArrayList<SMVTransition> transitions;
	
	private ArrayList<SMVInvariant> invariants;
	
	
	public SMVModule( String name )
	{
		this.name = name;
		
		variables = new ArrayList();
		
		assignments = new ArrayList();
		
		definitions = new ArrayList();
		
		transitions = new ArrayList();
		
		invariants = new ArrayList();
	}
	
	public String getName()
	{
		return name;
	}
	
	// --- METODI PER AGGIUNGERE ELEMENTI ---
	
	public void addVariable( SMVVariable variable )
	{
		variables.add( variable );
	}
	
	public void addAssignment( SMVAssignment assignment )
	{
		// Aggiungiamo una assegnazione alla lista solo se essa non  gi presente.
		// Se non facessimo cos, potremmo ottenere delle assegnazioni duplicate.
		
		if ( assignments.indexOf( assignment ) == -1 )
			
			assignments.add( assignment );
	}
	
	public void addDefinition( SMVDefinition definition )
	{
		definitions.add( definition );
	}
	
	public void addTransition( SMVTransition transition )
	{
		transitions.add( transition );
	}
	
	public void addInvariant( SMVInvariant invariant )
	{
		// Aggiungiamo un invariante alla lista solo se esso non  gi presente.
		// Se non facessimo cos, potremmo ottenere degli invarianti duplicati.
		
		if ( invariants.indexOf( invariant ) == -1 )
		
			invariants.add( invariant );
	}
	
	// --------------------------------------
	
	// --- METODI PER CONTARE ELEMENTI ---
	
	public int getVariableCount()
	{
		return variables.size();
	}
	
	public int getAssignmentCount()
	{
		return assignments.size();
	} 
		
	public int getDefinitionCount()
	{
		return definitions.size();
	}
	
	public int getTransitionCount()
	{
		return transitions.size();
	}
	
	public int getInvariantCount()
	{
		return invariants.size();
	}
	
	// -----------------------------------
	
	// --- METODI PER ESTRARRE ELEMENTI ---
	
	public SMVVariable getVariable( int index )
	{
		if ( index >= 0 && index < variables.size() )
			
			return variables.get( index );
			
		else
			
			return null;
	}
	
	public SMVAssignment getAssignment( int index )
	{
		if ( index >= 0 && index < assignments.size() )
			
			return assignments.get( index );
			
		else
			
			return null;
	}
	
	public SMVDefinition getDefinition( int index )
	{
		if ( index >= 0 && index < definitions.size() )
			
			return definitions.get( index );
			
		else
			
			return null;
	}
	
	public SMVTransition getTransition( int index )
	{
		if ( index >= 0 && index < transitions.size() )
			
			return transitions.get( index );
			
		else
			
			return null;
	}
	
	public SMVInvariant getInvariant( int index )
	{
		if ( index >= 0 && index < invariants.size() )
			
			return invariants.get( index );
			
		else
			
			return null;
	}
	
	// ------------------------------------	  	 

	public boolean equals( Object obj )
	{
		// ASSUNZIONE: due moduli sono uguali se e solo se hanno lo stesso nome.
		
		if ( obj != null & obj.getClass().equals( getClass() ) )
		{
			if ( ( ( SMVModule )( obj ) ).getName().equals( name ) )
				
				return true;
			
			else
					
				return false;
		}
		
		else
		
			return false;
	}
}